package org.fax4j.bridge.email;

import javax.mail.Message;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.bridge.FaxBridgeImpl;
import org.fax4j.bridge.FileInfo;

/**
 * This class provides partial implementation of the HTTP fax bridge.
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.vendor.policy.class.name</td>
 * 		<td>The vendor policy class name.</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.mail.message.parser.class.name</td>
 * 		<td>The mail message parser class name used to convert the mail message to fax job data.</td>
 * 		<td>org.fax4j.bridge.email.DefaultMailMessageParser</td>
 * 		<td>org.fax4j.bridge.email.DefaultMailMessageParser</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.0
 * @since	0.41.3
 */
public abstract class AbstractEMail2FaxBridge extends FaxBridgeImpl implements EMail2FaxBridge
{
	/**
	 * This is the class constructor.
	 */
	public AbstractEMail2FaxBridge()
	{
		super();
	}

	/**
	 * This function will submit a new fax job.
	 * 
	 * @param	mailMessage
	 * 			The mail message holding the fax job information
	 * @return	The fax job
	 */
	public FaxJob submitFaxJob(Message mailMessage)
	{
		if(mailMessage==null)
		{
			throw new FaxException("Mail message not provided.");
		}
		
		//create fax job
		FaxJob faxJob=this.getFaxJobFromMailMessage(mailMessage);
		
		//create file info
		FileInfo fileInfo=this.getFileInfoFromMailMessage(mailMessage);
		if(fileInfo==null)
		{
			throw new FaxException("Unable to extract file info from input mail message.");
		}
		
		//submit fax job
		this.submitFaxJob(faxJob,fileInfo);
		
		return faxJob;
	}
	
	/**
	 * This function returns the fax job from the mail message.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	mailMessage
	 * 			The mail message holding the fax job information
	 * @return	The fax job
	 */
	protected final FaxJob getFaxJobFromMailMessage(Message mailMessage)
	{
		//create fax job
		FaxJob faxJob=this.createFaxJob();
		
		//update fax job
		this.updateFaxJobFromMailMessage(mailMessage,faxJob);
		
		return faxJob;
	}
	
	/**
	 * This function returns the file info from the mail message.
	 * 
	 * @param	mailMessage
	 * 			The mail message holding the fax job information
	 * @return	The file info
	 */
	protected abstract FileInfo getFileInfoFromMailMessage(Message mailMessage);

	/**
	 * This function update the fax job from the mail message.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	mailMessage
	 * 			The mail message holding the fax job information
	 * @param	faxJob
	 * 			The fax job to update
	 */
	protected abstract void updateFaxJobFromMailMessage(Message mailMessage,FaxJob faxJob);
}