package org.fax4j.bridge.email;

import java.util.Map;
import javax.mail.Message;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.bridge.FileInfo;

/**
 * This is the email2fax based implementation of the fax bridge.
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.vendor.policy.class.name</td>
 * 		<td>The vendor policy class name.</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.mail.message.parser.class.name</td>
 * 		<td>The mail message parser class name used to convert the mail message to fax job data.</td>
 * 		<td>org.fax4j.bridge.email.DefaultMailMessageParser</td>
 * 		<td>org.fax4j.bridge.email.DefaultMailMessageParser</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.02
 * @since	0.40.6
 */
public class EMail2FaxBridgeImpl extends AbstractEMail2FaxBridge
{
	/**The mail message parser*/
	private MailMessageParser mailMessageParser;

	/**
	 * This is the class constructor.
	 */
	public EMail2FaxBridgeImpl()
	{
		super();
	}

	/**
	 * This function initializes the fax bridge.
	 */
	@Override
	protected void initializeImpl()
	{
		//create parser
		this.mailMessageParser=this.createMailMessageParser();
		if(this.mailMessageParser==null)
		{
			throw new FaxException("Unable to create mail message parser.");
		}
	}
	
	/**
	 * This function creates, initializes and returns a new mail message parser.
	 * 
	 * @return	The new mail message parser
	 */
	protected MailMessageParser createMailMessageParser()
	{
		//get class name
		String className=this.getConfigurationValue(EMail2FaxBridge.MAIL_MESSAGE_PARSER_CLASS_NAME_PROPERTY_KEY);

		if(className==null)
		{
			className=DefaultMailMessageParser.class.getName();
		}

		MailMessageParser parser=null;
		try
		{
			//create new instance
			Class<?> classDefinition=Class.forName(className);
			parser=(MailMessageParser)classDefinition.newInstance();
			
			//initialize
			Map<String,String> configuration=this.getConfiguration();
			parser.initialize(configuration);
		}
		catch(Exception exception)
		{
			throw new FaxException("Unable to create a new mail message parser for class name: "+className,exception);
		}
		
		return parser;
	}

	/**
	 * This function returns the file info from the request data.
	 * 
	 * @param	mailMessage
	 * 			The mail message holding the fax job information
	 * @return	The file info
	 */
	@Override
	protected FileInfo getFileInfoFromMailMessage(Message mailMessage)
	{
		return this.mailMessageParser.getFileInfoFromInputData(mailMessage);
	}

	/**
	 * This function update the fax job from the request data.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	mailMessage
	 * 			The mail message holding the fax job information
	 * @param	faxJob
	 * 			The fax job to update
	 */
	@Override
	protected void updateFaxJobFromMailMessage(Message mailMessage,FaxJob faxJob)
	{
		this.mailMessageParser.updateFaxJobFromInputData(mailMessage,faxJob);
	}
}