package org.fax4j.bridge.http;

import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.bridge.FaxBridgeImpl;
import org.fax4j.bridge.FileInfo;
import org.fax4j.spi.http.HTTPRequest;

/**
 * This class provides partial implementation of the HTTP fax bridge.
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.vendor.policy.class.name</td>
 * 		<td>The vendor policy class name.</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.http.request.parser.class.name</td>
 * 		<td>The HTTP request parser class name used to convert the HTTP request to fax job data.</td>
 * 		<td>org.fax4j.bridge.http.DefaultHTTPRequestParser</td>
 * 		<td>org.fax4j.bridge.http.DefaultHTTPRequestParser</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.0
 * @since	0.40.6
 */
public abstract class AbstractHTTP2FaxBridge extends FaxBridgeImpl implements HTTP2FaxBridge
{
	/**
	 * This is the class constructor.
	 */
	public AbstractHTTP2FaxBridge()
	{
		super();
	}

	/**
	 * This function will submit a new fax job.
	 * 
	 * @param	request
	 * 			The HTTP request
	 * @return	The submitted fax job
	 */
	public FaxJob submitFaxJob(HTTPRequest request)
	{
		if(request==null)
		{
			throw new FaxException("HTTP request not provided.");
		}
		
		//create fax job
		FaxJob faxJob=this.getFaxJobFromRequest(request);
		
		//create file info
		FileInfo fileInfo=this.getFileInfoFromRequest(request);
		if(fileInfo==null)
		{
			throw new FaxException("Unable to extract file info from input HTTP request.");
		}
		
		//submit fax job
		this.submitFaxJob(faxJob,fileInfo);
		
		return faxJob;
	}
	
	/**
	 * This function returns the fax job from the request data.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	request
	 * 			The HTTP request
	 * @return	The fax job
	 */
	protected final FaxJob getFaxJobFromRequest(HTTPRequest request)
	{
		//create fax job
		FaxJob faxJob=this.createFaxJob();
		
		//update fax job
		this.updateFaxJobFromRequest(request,faxJob);
		
		return faxJob;
	}
	
	/**
	 * This function returns the file info from the request data.
	 * 
	 * @param	request
	 * 			The HTTP request
	 * @return	The file info
	 */
	protected abstract FileInfo getFileInfoFromRequest(HTTPRequest request);

	/**
	 * This function update the fax job from the request data.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	request
	 * 			The HTTP request
	 * @param	faxJob
	 * 			The fax job to update
	 */
	protected abstract void updateFaxJobFromRequest(HTTPRequest request,FaxJob faxJob);
}