package org.fax4j.bridge.process;

import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.bridge.FaxBridgeImpl;
import org.fax4j.bridge.FileInfo;

/**
 * This class provides partial implementation of the process fax bridge.<br>
 * Process based bridges can be exposed as standalone java applications.
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.vendor.policy.class.name</td>
 * 		<td>The vendor policy class name.</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.process.command.parser.class.name</td>
 * 		<td>The command line parser class name used to convert the command line arguments to fax job data.</td>
 * 		<td>org.fax4j.bridge.process.DefaultCommandLineArgumentsParser</td>
 * 		<td>org.fax4j.bridge.process.DefaultCommandLineArgumentsParser</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.0
 * @since	0.40.6
 */
public abstract class AbstractProcess2FaxBridge extends FaxBridgeImpl implements Process2FaxBridge
{
	/**
	 * This is the class constructor.
	 */
	public AbstractProcess2FaxBridge()
	{
		super();
	}

	/**
	 * This function will submit a new fax job.
	 * 
	 * @param	commandLineArguments
	 * 			The command line arguments
	 * @return	The submitted fax job
	 */
	public FaxJob submitFaxJob(String[] commandLineArguments)
	{
		if((commandLineArguments==null)||(commandLineArguments.length==0))
		{
			throw new FaxException("Command line arguments not provided.");
		}
		
		//create fax job
		FaxJob faxJob=this.getFaxJobFromArguments(commandLineArguments);
		
		//create file info
		FileInfo fileInfo=this.getFileInfoFromArguments(commandLineArguments);
		if(fileInfo==null)
		{
			throw new FaxException("Unable to extract file info from input command line arguments.");
		}
		
		//submit fax job
		this.submitFaxJob(faxJob,fileInfo);
		
		return faxJob;
	}
	
	/**
	 * This function returns the fax job from the input data.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	commandLineArguments
	 * 			The command line arguments
	 * @return	The fax job
	 */
	protected final FaxJob getFaxJobFromArguments(String[] commandLineArguments)
	{
		//create fax job
		FaxJob faxJob=this.createFaxJob();
		
		//update fax job
		this.updateFaxJobFromArguments(commandLineArguments,faxJob);
		
		return faxJob;
	}
	
	/**
	 * This function returns the file info from the input data.
	 * 
	 * @param	commandLineArguments
	 * 			The command line arguments
	 * @return	The file info
	 */
	protected abstract FileInfo getFileInfoFromArguments(String[] commandLineArguments);

	/**
	 * This function update the fax job from the input data.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	commandLineArguments
	 * 			The command line arguments
	 * @param	faxJob
	 * 			The fax job to update
	 */
	protected abstract void updateFaxJobFromArguments(String[] commandLineArguments,FaxJob faxJob);
}