package org.fax4j.bridge.process;

import java.util.Map;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.bridge.FileInfo;

/**
 * This is the process cli2fax based implementation of the fax bridge.
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.vendor.policy.class.name</td>
 * 		<td>The vendor policy class name.</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>org.fax4j.bridge.EmptyVendorPolicy</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.bridge.process.command.parser.class.name</td>
 * 		<td>The command line parser class name used to convert the command line arguments to fax job data.</td>
 * 		<td>org.fax4j.bridge.process.DefaultCommandLineArgumentsParser</td>
 * 		<td>org.fax4j.bridge.process.DefaultCommandLineArgumentsParser</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.0
 * @since	0.40.6
 */
public class Process2FaxBridgeImpl extends AbstractProcess2FaxBridge
{
	/**The command line arguments parser*/
	private CommandLineArgumentsParser commandLineArgumentsParser;

	/**
	 * This is the class constructor.
	 */
	public Process2FaxBridgeImpl()
	{
		super();
	}

	/**
	 * This function initializes the fax bridge.
	 */
	@Override
	protected void initializeImpl()
	{
		//create parser
		this.commandLineArgumentsParser=this.createCommandLineArgumentsParser();
		if(this.commandLineArgumentsParser==null)
		{
			throw new FaxException("Unable to create command line arguments parser.");
		}
	}
	
	/**
	 * This function creates, initializes and returns a new command line arguments parser.
	 * 
	 * @return	The new command line arguments parser
	 */
	protected CommandLineArgumentsParser createCommandLineArgumentsParser()
	{
		//get class name
		String className=this.getConfigurationValue(Process2FaxBridge.COMMAND_PARSER_CLASS_NAME_PROPERTY_KEY);

		if(className==null)
		{
			className=DefaultCommandLineArgumentsParser.class.getName();
		}

		CommandLineArgumentsParser parser=null;
		try
		{
			//create new instance
			Class<?> classDefinition=Class.forName(className);
			parser=(CommandLineArgumentsParser)classDefinition.newInstance();
			
			//initialize
			Map<String,String> configuration=this.getConfiguration();
			parser.initialize(configuration);
		}
		catch(Exception exception)
		{
			throw new FaxException("Unable to create a new command line arguments parser for class name: "+className,exception);
		}
		
		return parser;
	}
	
	/**
	 * This function returns the file info from the input data.
	 * 
	 * @param	commandLineArguments
	 * 			The command line arguments
	 * @return	The file info
	 */
	@Override
	protected FileInfo getFileInfoFromArguments(String[] commandLineArguments)
	{
		return this.commandLineArgumentsParser.getFileInfoFromInputData(commandLineArguments);
	}

	/**
	 * This function update the fax job from the input data.<br>
	 * This fax job will not have any file data.
	 * 
	 * @param	commandLineArguments
	 * 			The command line arguments
	 * @param	faxJob
	 * 			The fax job to update
	 */
	@Override
	protected void updateFaxJobFromArguments(String[] commandLineArguments,FaxJob faxJob)
	{
		this.commandLineArgumentsParser.updateFaxJobFromInputData(commandLineArguments,faxJob);
	}
}