package org.fax4j.spi.http;

import java.io.InputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.util.IOHelper;
import org.fax4j.util.SpiUtil;

/**
 * This class implements the fax client service provider interface.<br>
 * This implementation will invoke the requests by sending HTTP requests to a web server that supports
 * sending fax messages.<br>
 * This class will use request templates to construct the HTTP requests.<br> 
 * Only the HTTP resource, URL parameters and HTTP content accept template parameters.<br>
 * The HTTP resource is the URL part after the domain, for example http://www.mydomain.com/<b>myresource</b>.<br>
 * The HTTP URL parameters is the URL part after the ?, for example http://www.mydomain.com/myresource?<b>param1=value1</b>.<br>
 * The HTTP content is not part of the URL and contains the POST information.<br>
 * The template parameters are than updated with the values from the fax4j configuration or FaxJob data.<br>
 * All template parameters are defined as ${templatename} varaiables, for example: http://www.mydomain.com/myresource?faxnum=<b>${target.address}</b><br>
 * <br>
 * Below table describes the template parameters.<br>
 * <b>Template Parameters:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Template Parameter</td>
 * 		<td>Description</td>
 * </tr>
 * 	<tr>
 * 		<td>file</td>
 * 		<td>The fax job file content (in case of template) or name (in case of resource or URL parameters)</td>
 * </tr>
 * 	<tr>
 * 		<td>target.address</td>
 * 		<td>The fax job target address</td>
 * </tr>
 * 	<tr>
 * 		<td>target.name</td>
 * 		<td>The fax job target name</td>
 * </tr>
 * 	<tr>
 * 		<td>sender.name</td>
 * 		<td>The fax job sender name</td>
 * </tr>
 * 	<tr>
 * 		<td>sender.fax.number</td>
 * 		<td>The fax job sender fax number</td>
 * </tr>
 * 	<tr>
 * 		<td>sender.email</td>
 * 		<td>The fax job sender email</td>
 * </tr>
 * </table>
 * <br>
 * In case there is no template defined for a certain fax action, this class will throw an UnsupportedOperationException exception.
 * <br>
 * The configuration of the fax4j framework is made up of 3 layers.<br>
 * The configuration is based on simple properties.<br>
 * Each layer overrides the lower layers by adding/changing the property values.<br>
 * The first layer is the internal fax4j.properties file located in the fax4j jar.<br>
 * This layer contains the preconfigured values for the fax4j framework and can be changed
 * by updating these properties in the higher layers.<br>
 * The second layer is the external fax4j.properties file that is located on the classpath.<br>
 * This file is optional and provides the ability to override the internal configuration for the
 * entire fax4j framework.<br>
 * The top most layer is the optional java.util.Properties object provided by the external classes
 * when creating a new fax client.<br>
 * These properties enable to override the configuration of the lower 2 layers.<br>
 * <br>
 * <b>SPI Status (Draft, Beta, Stable): </b>Beta<br>
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.client.class.name</td>
 * 		<td>The HTTP client class name used to submit the HTTP requests</td>
 * 		<td>org.fax4j.spi.http.ApacheHTTPClient</td>
 * 		<td>org.fax4j.spi.http.ApacheHTTPClient</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.host.name</td>
 * 		<td>The target (service provider) host name.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>true</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.port</td>
 * 		<td>The target port number.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.ssl</td>
 * 		<td>True to use SSL (HTTPS), false for simple HTTP</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.resource</td>
 * 		<td>The default HTTP resource of the request</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.url.parameters</td>
 * 		<td>The default HTTP URL parameters of the request</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.template.encoding</td>
 * 		<td>The template encoding, if encoding is not defined, the default system encoding is used.</td>
 * 		<td>none</td>
 * 		<td>The default system encoding</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.submit.template.url</td>
 * 		<td>The URL pointing to the submit fax job template.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.suspend.template.url</td>
 * 		<td>The URL pointing to the suspend fax job template.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.resume.template.url</td>
 * 		<td>The URL pointing to the resume fax job template.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.cancel.template.url</td>
 * 		<td>The URL pointing to the cancel fax job template.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * </table>
 * <br>
 * <b>Limitations:</b><br>
 * <ul>
 * 	<li>Fax Job is not updated with HTTP output.
 * 	<li>HTTP request content supports only text content.
 * 	<li>HTTP response is only validated for status code.
 * </ul>
 * <br>
 * <b>Dependencies:</b><br>
 * <ul>
 * 	<li>HTTP client required jars.
 * </ul>
 * <br>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.06
 * @since	0.21a
 */
public class HTTPTemplateFaxClientSpi extends AbstractHTTPFaxClientSpi
{
	/**The HTTP request template map*/
	private Map<String,String> httpRequestTemplateMap;

	/**
	 * This class holds the SPI configuration constants.
	 * 
	 * @author 	Sagie Gur-Ari
	 * @version 1.01
	 * @since	0.21a
	 */
	public static class FaxClientSpiConfigurationConstants extends AbstractHTTPFaxClientSpi.FaxClientSpiConfigurationConstants
	{
		/**
		 * This is the class constructor.
		 */
		private FaxClientSpiConfigurationConstants()
		{
			super();
		}

		/**The template encoding property key*/
		public static final String TEMPLATE_ENCODING_PROPERTY_KEY="org.fax4j.spi.http.template.encoding";
		/**The submit fax job template URL property key*/
		public static final String SUBMIT_FAX_JOB_TEMPLATE_PROPERTY_KEY="org.fax4j.spi.http.submit.template.url";
		/**The suspend fax job template URL property key*/
		public static final String SUSPEND_FAX_JOB_TEMPLATE_PROPERTY_KEY="org.fax4j.spi.http.suspend.template.url";
		/**The resume fax job template URL property key*/
		public static final String RESUME_FAX_JOB_TEMPLATE_PROPERTY_KEY="org.fax4j.spi.http.resume.template.url";
		/**The cancel fax job template URL property key*/
		public static final String CANCEL_FAX_JOB_TEMPLATE_PROPERTY_KEY="org.fax4j.spi.http.cancel.template.url";
	}

	/**
	 * This is the default constructor.
	 */
	public HTTPTemplateFaxClientSpi()
	{
		super();
	}
	
	/**
	 * This function initializes the fax client SPI.
	 */
	@Override
	protected void initializeImpl()
	{
		//initialize
		super.initializeImpl();
		
		//init templates map
		this.httpRequestTemplateMap=new HashMap<String,String>();
		
		String[] templateNames=new String[]
		{
			FaxClientSpiConfigurationConstants.SUBMIT_FAX_JOB_TEMPLATE_PROPERTY_KEY,
			FaxClientSpiConfigurationConstants.SUSPEND_FAX_JOB_TEMPLATE_PROPERTY_KEY,
			FaxClientSpiConfigurationConstants.RESUME_FAX_JOB_TEMPLATE_PROPERTY_KEY,
			FaxClientSpiConfigurationConstants.CANCEL_FAX_JOB_TEMPLATE_PROPERTY_KEY
		};
		
		//get templates amount
		int amount=templateNames.length;
		
		//get template encoding value
		String encoding=this.getConfigurationValue(FaxClientSpiConfigurationConstants.TEMPLATE_ENCODING_PROPERTY_KEY);
		
		String templateName=null;
		String urlStr=null;
		URL url=null;
		String template=null;
		for(int index=0;index<amount;index++)
		{
			//get next element
			templateName=templateNames[index];
			
			//get URL string
			urlStr=this.getConfigurationValue(templateName);
			
			if(urlStr!=null)
			{
				try
				{
					//create URL
					url=new URL(urlStr);
					
					//read template
					InputStream stream=url.openStream();
					Reader reader=IOHelper.createReader(stream,encoding);
					template=IOHelper.readTextStream(reader);
					
					//cache template
					this.httpRequestTemplateMap.put(templateName,template);
				}
				catch(Throwable throwable)
				{
					throw new FaxException("Unable to load template for URL: "+urlStr,throwable);
				}
			}
		}
	}
	
	/**
	 * This function formats the provided template.
	 * 
	 * @param 	template
	 * 			The template
	 * @param 	faxJob
	 * 			The fax job object
	 * @return	The formatted template
	 */
	protected String formatTemplate(String template,FaxJob faxJob)
	{
		return SpiUtil.formatTemplate(template,faxJob,null,true,false);
	}
	
	/**
	 * This function formats the HTTP resource.
	 * 
	 * @param 	faxJob
	 * 			The fax job object
	 * @return	The formatted HTTP resource
	 */
	protected String formatHTTPResource(FaxJob faxJob)
	{
		//get resource
		String resourceTemplate=this.getHTTPResource();
		
		//format resource
		String resource=SpiUtil.formatTemplate(resourceTemplate,faxJob,SpiUtil.URL_ENCODER,false,false);
		
		return resource;
	}

	/**
	 * This function formats the HTTP URL parameters.
	 * 
	 * @param 	faxJob
	 * 			The fax job object
	 * @return	The formatted HTTP URL parameters
	 */
	protected String formatHTTPURLParameters(FaxJob faxJob)
	{
		//get URL parameters
		String urlParametersTemplate=this.getHTTPURLParameters();
		
		//format URL parameters
		String urlParameters=SpiUtil.formatTemplate(urlParametersTemplate,faxJob,SpiUtil.URL_ENCODER,false,false);
		
		return urlParameters;
	}
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param	templateName
	 * 			The template name
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	protected HTTPRequest createHTTPRequest(String templateName,FaxJob faxJob)
	{
		//get template
		String template=this.httpRequestTemplateMap.get(templateName);

		HTTPRequest httpRequest=null;
		if(template!=null)
		{
			//get encoding
			String defaultValue=IOHelper.getDefaultEncoding();
			String encoding=faxJob.getProperty(FaxJobExtendedPropertyConstants.TEXT_FILE_ENCODING_FAX_JOB_PROPERTY_KEY,defaultValue);
			
			//format template
			String contentStr=this.formatTemplate(template,faxJob);
			byte[] content=null;
			try
			{
				content=contentStr.getBytes(encoding);
			}
			catch(UnsupportedEncodingException exception)
			{
				throw new FaxException("Unable to convert content to bytes, unsupported encoding: "+encoding,exception);
			}

			//format resource
			String resource=this.formatHTTPResource(faxJob);
			
			//format URL parameters
			String parametersText=this.formatHTTPURLParameters(faxJob);
			
			//create HTTP request
			httpRequest=new HTTPRequest();
			
			//setup HTTP request
			httpRequest.setResource(resource);
			httpRequest.setContent(content);
			httpRequest.setParametersText(parametersText);
		}

		return httpRequest;
	}
	
	/**
	 * Updates the fax job based on the data from the HTTP response.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	httpResponse
	 * 			The HTTP response
	 */
	@Override
	protected void updateFaxJob(FaxJob faxJob,HTTPResponse httpResponse)
	{
		//empty implementation
	}
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	@Override
	protected HTTPRequest createSubmitFaxJobHTTPRequest(FaxJob faxJob)
	{
		return this.createHTTPRequest(FaxClientSpiConfigurationConstants.SUBMIT_FAX_JOB_TEMPLATE_PROPERTY_KEY,faxJob);
	}
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	@Override
	protected HTTPRequest createSuspendFaxJobHTTPRequest(FaxJob faxJob)
	{
		return this.createHTTPRequest(FaxClientSpiConfigurationConstants.SUSPEND_FAX_JOB_TEMPLATE_PROPERTY_KEY,faxJob);
	}
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	@Override
	protected HTTPRequest createResumeFaxJobHTTPRequest(FaxJob faxJob)
	{
		return this.createHTTPRequest(FaxClientSpiConfigurationConstants.RESUME_FAX_JOB_TEMPLATE_PROPERTY_KEY,faxJob);
	}
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	@Override
	protected HTTPRequest createCancelFaxJobHTTPRequest(FaxJob faxJob)
	{
		return this.createHTTPRequest(FaxClientSpiConfigurationConstants.CANCEL_FAX_JOB_TEMPLATE_PROPERTY_KEY,faxJob);
	}
}