package org.fax4j.spi.windows;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.FaxJobStatus;
import org.fax4j.common.FaxActionType;
import org.fax4j.common.LogLevel;
import org.fax4j.common.Logger;
import org.fax4j.spi.AbstractFax4JClientSpi;
import org.fax4j.spi.process.ProcessOutputValidator;
import org.fax4j.util.IOHelper;
import org.fax4j.util.ProcessExecutorHelper;
import org.fax4j.util.ProcessExecutorHelper.ProcessOutput;

/**
 * This class implements the fax client service provider interface.<br>
 * This implementation will proxy the requests to windows native fax API.<br>
 * It is possible to see all faxes submitted from this SPI in the windows fax window available from the
 * Start->Settings->Printers and Faxes->Fax
 * <br>
 * The configuration of the fax4j framework is made up of 3 layers.<br>
 * The configuration is based on simple properties.<br>
 * Each layer overrides the lower layers by adding/changing the property values.<br>
 * The first layer is the internal fax4j.properties file located in the fax4j jar.<br>
 * This layer contains the preconfigured values for the fax4j framework and can be changed
 * by updating these properties in the higher layers.<br>
 * The second layer is the external fax4j.properties file that is located on the classpath.<br>
 * This file is optional and provides the ability to override the internal configuration for the
 * entire fax4j framework.<br>
 * The top most layer is the optional java.util.Properties object provided by the external classes
 * when creating a new fax client.<br>
 * These properties enable to override the configuration of the lower 2 layers.<br>
 * <br>
 * <b>SPI Status (Draft, Beta, Stable): </b>Stable<br>
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.windows.server.name</td>
 * 		<td>The fax server host name.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.windows.use.jni</td>
 * 		<td>True to use JNI, false to use the external fax4j.exe</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * <br>
 * <b>Limitations:</b><br>
 * <ul>
 * 	<li>This SPI will only work on Windows OS (windows 2000 and up).
 * 	<li>See windows fax/tapi limitations.
 * 	<li>JNI set to true only supports 32 bit JVMs (Windows OS can be 64 bit).
 * </ul>
 * <br>
 * <b>Dependencies:</b><br>
 * <ul>
 * 	<li>Required native libraries: winfax.dll
 * 	<li>Windows OS (windows 2000 and up) - see http://msdn.microsoft.com/en-us/library/ms692855(VS.85).aspx for more info.
 * 	<li>Windows fax component installed.
 * </ul>
 * <br>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.12
 * @since	0.1
 */
public class WindowsFaxClientSpi extends AbstractFax4JClientSpi
{
	/**The process output validator*/
	private ProcessOutputValidator processOutputValidator;
	/**The fax server name*/
	private String faxServerName;
	/**The use JNI flag*/
	private boolean useJNI;
	/**The fax4j executable file location*/
	private String fax4jExecutableFileLocation;
	/**The loaded native library flag*/
	private static boolean nativeLibraryLoaded=false;
	/**The lock used to synchronize native calls*/
	private static final Object NATIVE_LOCK=new Object();
	
	/**
	 * This class holds the SPI configuration constants.
	 * 
	 * @author 	Sagie Gur-Ari
	 * @version 1.01
	 * @since	0.1
	 */
	public static class FaxClientSpiConfigurationConstants
	{
		/**
		 * This is the class constructor.
		 */
		private FaxClientSpiConfigurationConstants()
		{
			super();
		}

		/**The fax server name property key*/
		public static final String FAX_SERVER_NAME_PROPERTY_KEY="org.fax4j.spi.windows.server.name";
		/**The use JNI flag property key*/
		public static final String USE_JNI_PROPERTY_KEY="org.fax4j.spi.windows.use.jni";
	}

	/**
	 * This class holds the fax4j exe constants.
	 * 
	 * @author 	Sagie Gur-Ari
	 * @version 1.0
	 * @since	0.22
	 */
	public static class Fax4jExeConstants
	{
		/**
		 * This is the class constructor.
		 */
		private Fax4jExeConstants()
		{
			super();
		}

		/**The fax4j.exe*/
		public static final String FAX4J_EXE="fax4j.exe";
		/**The space string*/
		public static final String SPACE_STR=" ";
		/**The value wrapper string*/
		public static final String VALUE_WRAPPER="\"";
		/**fax4j.exe command line argument constant*/
		public static final String SUBMIT_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE="submit";
		/**fax4j.exe command line argument constant*/
		public static final String SUSPEND_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE="suspend";
		/**fax4j.exe command line argument constant*/
		public static final String RESUME_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE="resume";
		/**fax4j.exe command line argument constant*/
		public static final String CANCEL_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE="cancel";
		/**fax4j.exe command line argument constant*/
		public static final String GET_STATUS_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE="getstatus";
		/**fax4j.exe command line argument constant*/
		public static final String ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-action";
		/**fax4j.exe command line argument constant*/
		public static final String SERVER_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-server";
		/**fax4j.exe command line argument constant*/
		public static final String FAX_JOB_ID_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-job";
		/**fax4j.exe command line argument constant*/
		public static final String TARGET_ADDRESS_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-target_address";
		/**fax4j.exe command line argument constant*/
		public static final String TARGET_NAME_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-target_name";
		/**fax4j.exe command line argument constant*/
		public static final String SENDER_NAME_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-sender_name";
		/**fax4j.exe command line argument constant*/
		public static final String FILE_NAME_FAX4J_EXE_COMMAND_LINE_ARGUMENT="-file";
		/**The fax operation done text*/
		public static final String OPERATION_OUTPUT_DONE="fax.operation.done=true";
		/**The fax job ID output prefix*/
		public static final String FAX_JOB_ID_OUTPUT_PREFIX="fax.job.id=";
		/**The fax job ID output prefix length*/
		public static final int FAX_JOB_ID_OUTPUT_PREFIX_LENGTH=Fax4jExeConstants.FAX_JOB_ID_OUTPUT_PREFIX.length();
		/**The fax job status output prefix*/
		public static final String FAX_JOB_STATUS_OUTPUT_PREFIX="fax.job.status=";
		/**The fax job status output prefix length*/
		public static final int FAX_JOB_STATUS_OUTPUT_PREFIX_LENGTH=Fax4jExeConstants.FAX_JOB_STATUS_OUTPUT_PREFIX.length();
		/**The fax error output prefix*/
		public static final String FAX_ERROR_OUTPUT_PREFIX="fax.error=";
		/**The fax error output prefix length*/
		public static final int FAX_ERROR_OUTPUT_PREFIX_LENGTH=Fax4jExeConstants.FAX_ERROR_OUTPUT_PREFIX.length();
	}
	
	/**
	 * This is the default constructor.
	 */
	public WindowsFaxClientSpi()
	{
		super();
	}
	
	/**
	 * This function extracts the native resources (the fax4j.exe and fax4j.dll) 
	 * and pushes them to the fax4j temporary directory.
	 */
	private static void extractNativeResources()
	{
		synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
		{
			//get target directory
			File directory=IOHelper.getFax4jInternalTemporaryDirectory();
			
			//extract resources
			String[] names=new String[]{"fax4j.dll","fax4j.exe"};
			int amount=names.length;
			String name=null;
			File file=null;
			InputStream inputStream=null;
			OutputStream outputStream=null;
			for(int index=0;index<amount;index++)
			{
				//get next resource
				name=names[index];
				
				//get file
				file=new File(directory,name);
				
				if(!file.exists())
				{
					//get input stream
					inputStream=WindowsFaxClientSpi.class.getResourceAsStream(name);
					if(inputStream!=null)
					{
						try
						{
							//create output stream
							outputStream=new FileOutputStream(file);
							
							//write data to file
							IOHelper.readAndWriteStreams(inputStream,outputStream);
						}
						catch(IOException exception)
						{
							throw new FaxException("Unable to extract resource: "+name,exception);
						}
					}
				}
			}
		}
	}
	
	/**
	 * Loads the native library if not loaded before.
	 * 
	 * @param	logger
	 * 			The logger
	 */
	private static void loadNativeLibrary(Logger logger)
	{
		synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
		{
			if(!WindowsFaxClientSpi.nativeLibraryLoaded)
			{
				try
				{
					//get temporary directory
					File directory=IOHelper.getFax4jInternalTemporaryDirectory();

					//get dll path
					File dllFile=new File(directory,"fax4j.dll");

					//get path
					String path=dllFile.getPath();

					//load native library
					System.load(path);
					logger.logDebug(new Object[]{"Loaded native library runtime path."},null);
					
					//set flag
					WindowsFaxClientSpi.nativeLibraryLoaded=true;
				}
				catch(Throwable throwable)
				{
					logger.logError(new Object[]{"Error while loading native library from runtime path."},throwable);
				}

				if(!WindowsFaxClientSpi.nativeLibraryLoaded)
				{
					try
					{
						//load native library
						System.loadLibrary("fax4j");
						logger.logDebug(new Object[]{"Loaded native library from native path."},null);
						
						//set flag
						WindowsFaxClientSpi.nativeLibraryLoaded=true;
					}
					catch(Throwable throwable)
					{
						logger.logError(new Object[]{"Error while loading native library from native path."},throwable);
					}
				}
			}
		}
	}
	
	/**
	 * This function initializes the fax client SPI.
	 */
	@Override
	protected void initializeImpl()
	{
		//get logger
		Logger logger=this.getLogger();
		
		//init process output validator
		this.processOutputValidator=new WindowsProcessOutputValidator();
		
		//get fax server name
		this.faxServerName=this.getConfigurationValue(FaxClientSpiConfigurationConstants.FAX_SERVER_NAME_PROPERTY_KEY);
		logger.logDebug(new Object[]{"Fax server name: ",this.faxServerName},null);
		
		//get use JNI flag
		this.useJNI=false;
		String useJNIStr=this.getConfigurationValue(FaxClientSpiConfigurationConstants.USE_JNI_PROPERTY_KEY);
		if(useJNIStr!=null)
		{
			this.useJNI=Boolean.parseBoolean(useJNIStr);
		}
		
		//extract native resources
		WindowsFaxClientSpi.extractNativeResources();
		
		if(this.useJNI)
		{
			//load native library
			WindowsFaxClientSpi.loadNativeLibrary(logger);
			
			if(!WindowsFaxClientSpi.nativeLibraryLoaded)
			{
				//if failed to load native DLL, use executable
				this.useJNI=false;
			}
		}
		
		//if use JNI set to false or if failed to load native DLL (possible jvm 64 bit)
		if(!this.useJNI)
		{
			//get temporary directory
			File directory=IOHelper.getFax4jInternalTemporaryDirectory();

			//get exe path
			File exeFile=new File(directory,Fax4jExeConstants.FAX4J_EXE);
			if(exeFile.exists())
			{
				this.fax4jExecutableFileLocation=exeFile.getPath();
			}
			else
			{
				exeFile=IOHelper.getFileFromNativePath(Fax4jExeConstants.FAX4J_EXE);
				if(exeFile!=null)
				{
					this.fax4jExecutableFileLocation=exeFile.getPath();
				}
			}
		}
	}
	
	/**
	 * This function is invoked before any native call to set the
	 * native layer debug mode.
	 */
	protected void preNativeCall()
	{
		//get logger
		Logger logger=this.getLogger();
		
		//get log level
		LogLevel logLevel=logger.getLogLevel();
		
		boolean debugMode=false;
		if(logLevel.equals(LogLevel.DEBUG))
		{
			debugMode=true;
		}
		
		//set debug mode
		WindowsFaxClientSpi.setDebugModeNative(debugMode);
	}
	
	/**
	 * This function adds the given command line argument to the buffer.
	 * 
	 * @param 	buffer
	 * 			The buffer
	 * @param 	argument
	 * 			The argument
	 * @param 	value
	 * 			The argument value
	 */
	protected void addCommandLineArgument(StringBuilder buffer,String argument,String value)
	{
		if((value!=null)&&(value.length()>0))
		{
			buffer.append(argument);
			buffer.append(Fax4jExeConstants.SPACE_STR);
			buffer.append(Fax4jExeConstants.VALUE_WRAPPER);
			buffer.append(value);
			buffer.append(Fax4jExeConstants.VALUE_WRAPPER);
			buffer.append(Fax4jExeConstants.SPACE_STR);
		}
	}
	
	/**
	 * This function creates and returns the fax4j.exe command.
	 * 
	 * @param	commandArguments
	 * 			The command line arguments
	 * @return	The fax4j.exe command
	 */
	protected String createProcessCommand(String commandArguments)
	{
		//create command
		StringBuilder buffer=new StringBuilder(500);
		buffer.append("\"");
		buffer.append(this.fax4jExecutableFileLocation);
		buffer.append("\"");
		buffer.append(Fax4jExeConstants.SPACE_STR);
		buffer.append(commandArguments);
		String command=buffer.toString();

		return command;
	}
	
	/**
	 * This function handles the fax4j.exe process output.
	 * Handling includes error validation and output extraction.
	 * 
	 * @param	faxActionType
	 * 			The fax action type
	 * @param 	processOutput
	 * 			The process output object
	 * @return	The process output value
	 */
	protected String handleProcessOutput(FaxActionType faxActionType,ProcessOutput processOutput)
	{
		//call output validator
		this.processOutputValidator.validateProcessOutput(this,processOutput,faxActionType);

		//get output
		String output=processOutput.getOutputText();

		if(output!=null)
		{
			//if output is fax job ID
			int index=output.indexOf(Fax4jExeConstants.FAX_JOB_ID_OUTPUT_PREFIX);
			int prefixLenght=Fax4jExeConstants.FAX_JOB_ID_OUTPUT_PREFIX_LENGTH;
			if(index==-1)	//if output is fax job status
			{
				index=output.indexOf(Fax4jExeConstants.FAX_JOB_STATUS_OUTPUT_PREFIX);
				prefixLenght=Fax4jExeConstants.FAX_JOB_STATUS_OUTPUT_PREFIX_LENGTH;
			}
			if(index!=-1)
			{
				//get index
				index=index+prefixLenght;
	
				if(output.length()>index)
				{
					//get output
					output=output.substring(index);
					
					output=output.trim();
					index=output.indexOf("\n");
					if(index!=-1)
					{
						output=output.substring(0,index);
						output=output.trim();
					}
					
					//ignore empty output
					if(output.length()==0)
					{
						output=null;
					}
				}
				else
				{
					output=null;
				}
			}
			else
			{
				output=null;
			}
		}

		return output;
	}
	
	/**
	 * This function executes the fax4j.exe with the provided argument line.
	 * 
	 * @param	commandArguments
	 * 			The command line arguments
	 * @param	faxActionType
	 * 			The fax action type
	 * @return	The process output
	 */
	protected String executeProcess(FaxActionType faxActionType,String commandArguments)
	{
		//create command
		String command=this.createProcessCommand(commandArguments);

		ProcessOutput processOutput=null;
		try
		{
			//execute command
			processOutput=ProcessExecutorHelper.executeProcess(this,command);
		}
		catch(RuntimeException exception)
		{
			throw exception;
		}
		catch(Exception exception)
		{
			throw new FaxException("Error while invoking "+Fax4jExeConstants.FAX4J_EXE+" command: "+command,exception);
		}

		//handle output
		String output=this.handleProcessOutput(faxActionType,processOutput);

		return output;
	}
	
	/**
	 * This function validates the provided fax job ID (not checking
	 * fax server, only format).
	 * In case of an invalid ID, an error will be thrown.
	 * 
	 * @param 	faxJobID
	 * 			The fax job ID to validate
	 */
	protected void validateFaxJobID(String faxJobID)
	{
		if((faxJobID==null)||(faxJobID.length()==0))
		{
			throw new FaxException("Fax job ID not provided.");
		}
		int faxJobIDInt=Integer.parseInt(faxJobID);
		this.validateFaxJobID(faxJobIDInt);
	}
	
	/**
	 * This function validates the provided fax job ID (not checking
	 * fax server, only format).
	 * In case of an invalid ID, an error will be thrown.
	 * 
	 * @param 	faxJobID
	 * 			The fax job ID to validate
	 */
	protected void validateFaxJobID(int faxJobID)
	{
		if(faxJobID==0)
		{
			throw new FaxException("Invalid fax job ID provided: "+faxJobID);
		}
	}
	
	/**
	 * This function returns the fax job ID (if valid).
	 * If fax job ID is not valid, an error will be thrown.
	 * 
	 * @param 	faxJob
	 * 			The fax job object
	 * @return	The fax job ID
	 */
	protected int getFaxJobID(FaxJob faxJob)
	{
		//get fax job ID
		String faxJobID=faxJob.getID();
		this.validateFaxJobID(faxJobID);
		int faxJobIDInt=Integer.parseInt(faxJobID);

		return faxJobIDInt;
	}
	
	/**
	 * This function returns the fax job status based on the windows
	 * fax job status string value.
	 * 
	 * @param 	faxJobStatusStr
	 * 			The fax job status string value
	 * @return	The fax job status
	 */
	protected FaxJobStatus getFaxJobStatusFromWindowsFaxJobStatusString(String faxJobStatusStr)
	{
		FaxJobStatus faxJobStatus=FaxJobStatus.UNKNOWN;
		if((faxJobStatusStr!=null)&&(faxJobStatusStr.length()>0))
		{
			if((faxJobStatusStr.equalsIgnoreCase("JS_PENDING"))||(faxJobStatusStr.equalsIgnoreCase("JS_PAUSED"))||(faxJobStatusStr.equalsIgnoreCase("JS_RETRYING")))
			{
				faxJobStatus=FaxJobStatus.PENDING;
			}
			else if(faxJobStatusStr.equalsIgnoreCase("JS_INPROGRESS"))
			{
				faxJobStatus=FaxJobStatus.IN_PROGRESS;
			}
			else if((faxJobStatusStr.equalsIgnoreCase("JS_FAILED"))||(faxJobStatusStr.equalsIgnoreCase("JS_NOLINE"))||(faxJobStatusStr.equalsIgnoreCase("JS_RETRIES_EXCEEDED")))
			{
				faxJobStatus=FaxJobStatus.ERROR;
			}
		}
		
		return faxJobStatus;
	}
	
	/**
	 * This function creates and returns the command line arguments for the fax4j
	 * external exe when running the submit fax job action.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param	targetAddress
	 * 			The fax job target address
	 * @param	targetName
	 * 			The fax job target name
	 * @param	senderName
	 * 			The fax job sender name
	 * @param	fileName
	 * 			The file to fax
	 * @return	The full command line arguments line
	 */
	protected String createProcessCommandArgumentsForSubmitFaxJob(String serverName,String targetAddress,String targetName,String senderName,String fileName)
	{
		//init buffer
		StringBuilder buffer=new StringBuilder();
		
		//create command line arguments
		this.addCommandLineArgument(buffer,Fax4jExeConstants.ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT,Fax4jExeConstants.SUBMIT_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.SERVER_FAX4J_EXE_COMMAND_LINE_ARGUMENT,serverName);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.TARGET_ADDRESS_FAX4J_EXE_COMMAND_LINE_ARGUMENT,targetAddress);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.TARGET_NAME_FAX4J_EXE_COMMAND_LINE_ARGUMENT,targetName);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.SENDER_NAME_FAX4J_EXE_COMMAND_LINE_ARGUMENT,senderName);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.FILE_NAME_FAX4J_EXE_COMMAND_LINE_ARGUMENT,fileName);
		
		//get text
		String commandArguments=buffer.toString();
		
		return commandArguments;
	}
	
	/**
	 * This function creates and returns the command line arguments for the fax4j
	 * external exe when running an action on an existing fax job.
	 * 
	 * @param	faxActionTypeArgument
	 * 			The fax action type argument to be used in the command line
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 * @return	The full command line arguments line
	 */
	protected String createProcessCommandArgumentsForExistingFaxJob(String faxActionTypeArgument,String serverName,int faxJobID)
	{
		//init buffer
		StringBuilder buffer=new StringBuilder();
		
		//create command line arguments
		this.addCommandLineArgument(buffer,Fax4jExeConstants.ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT,faxActionTypeArgument);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.SERVER_FAX4J_EXE_COMMAND_LINE_ARGUMENT,serverName);
		this.addCommandLineArgument(buffer,Fax4jExeConstants.FAX_JOB_ID_FAX4J_EXE_COMMAND_LINE_ARGUMENT,String.valueOf(faxJobID));
		
		//get text
		String commandArguments=buffer.toString();
		
		return commandArguments;
	}
	
	/**
	 * This function will submit a new fax job.<br>
	 * The fax job ID may be populated by this method in the provided
	 * fax job object.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void submitFaxJobImpl(FaxJob faxJob)
	{
		//get fax job values
		String targetAddress=faxJob.getTargetAddress();
		String targetName=faxJob.getTargetName();
		if(targetName==null)
		{
			targetName="";
		}
		String senderName=faxJob.getSenderName();
		if(senderName==null)
		{
			senderName="";
		}
		File file=faxJob.getFile();
		String filePath=null;
		try
		{
			filePath=file.getCanonicalPath();
		}
		catch(Exception exception)
		{
			throw new FaxException("Unable to extract canonical path from file: "+file,exception);
		}
		
		//invoke fax action
		int faxJobIDInt=this.winSubmitFaxJob(this.faxServerName,targetAddress,targetName,senderName,filePath);
		
		//validate fax job ID
		this.validateFaxJobID(faxJobIDInt);
		
		//set fax job ID
		String faxJobID=String.valueOf(faxJobIDInt);
		faxJob.setID(faxJobID);
	}
	
	/**
	 * This function will suspend an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void suspendFaxJobImpl(FaxJob faxJob)
	{
		//get fax job ID
		int faxJobIDInt=this.getFaxJobID(faxJob);

		//invoke fax action
		this.winSuspendFaxJob(this.faxServerName,faxJobIDInt);
	}
	
	/**
	 * This function will resume an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void resumeFaxJobImpl(FaxJob faxJob)
	{
		//get fax job ID
		int faxJobIDInt=this.getFaxJobID(faxJob);

		//invoke fax action
		this.winResumeFaxJob(this.faxServerName,faxJobIDInt);
	}
	
	/**
	 * This function will cancel an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void cancelFaxJobImpl(FaxJob faxJob)
	{
		//get fax job ID
		int faxJobIDInt=this.getFaxJobID(faxJob);

		//invoke fax action
		this.winCancelFaxJob(this.faxServerName,faxJobIDInt);
	}
	
	/**
	 * This function returns the fax job status.<br>
	 * Not all SPIs support extraction of the fax job status.<br>
	 * In case the SPI is unable to extract or does not support extracting
	 * of the fax job status, it will return the UNKNOWN status.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 * @return	The fax job status
	 */
	@Override
	protected FaxJobStatus getFaxJobStatusImpl(FaxJob faxJob)
	{
		//get fax job ID
		int faxJobIDInt=this.getFaxJobID(faxJob);

		//invoke fax action
		String faxJobStatusStr=this.winGetFaxJobStatus(this.faxServerName,faxJobIDInt);

		//get fax job status
		FaxJobStatus faxJobStatus=this.getFaxJobStatusFromWindowsFaxJobStatusString(faxJobStatusStr);
		
		return faxJobStatus;
	}
	
	/**
	 * This function returns true if the fax monitor events are supported by this SPI.
	 * 
	 * @return	True if the fax monitor events are supported by this SPI
	 */
	@Override
	protected boolean isFaxMonitorEventsSupported()
	{
		return true;
	}
	
	/**
	 * This function will submit a new fax job.<br>
	 * The fax job ID will be returned by this method.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param	targetAddress
	 * 			The fax job target address
	 * @param	targetName
	 * 			The fax job target name
	 * @param	senderName
	 * 			The fax job sender name
	 * @param	fileName
	 * 			The file to fax
	 * @return	The fax job ID (null in case of an error)
	 */
	private int winSubmitFaxJob(String serverName,String targetAddress,String targetName,String senderName,String fileName)
	{
		int faxJobID=0;
		if(this.useJNI)
		{
			synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
			{
				//pre native call
				this.preNativeCall();
				
				//invoke native
				faxJobID=WindowsFaxClientSpi.submitFaxJobNative(serverName,targetAddress,targetName,senderName,fileName);
			}
		}
		else
		{
			//create command arguments
			String commandArguments=this.createProcessCommandArgumentsForSubmitFaxJob(serverName,targetAddress,targetName,senderName,fileName);

			//execute process
			String faxJobIDStr=this.executeProcess(FaxActionType.SUBMIT_FAX_JOB,commandArguments);
			
			//handle output
			if(faxJobIDStr!=null)
			{
				faxJobID=Integer.parseInt(faxJobIDStr);
			}
		}
		
		return faxJobID;
	}
	
	/**
	 * This function will suspend an existing fax job.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 */
	private void winSuspendFaxJob(String serverName,int faxJobID)
	{
		if(this.useJNI)
		{
			synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
			{
				//pre native call
				this.preNativeCall();
				
				//invoke native
				WindowsFaxClientSpi.suspendFaxJobNative(serverName,faxJobID);
			}
		}
		else
		{
			//create command arguments
			String commandArguments=this.createProcessCommandArgumentsForExistingFaxJob(Fax4jExeConstants.SUSPEND_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE,serverName,faxJobID);
			
			//execute process
			this.executeProcess(FaxActionType.SUSPEND_FAX_JOB,commandArguments);
		}
	}
	
	/**
	 * This function will resume an existing fax job.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 */
	private void winResumeFaxJob(String serverName,int faxJobID)
	{
		if(this.useJNI)
		{
			synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
			{
				//pre native call
				this.preNativeCall();
				
				//invoke native
				WindowsFaxClientSpi.resumeFaxJobNative(serverName,faxJobID);
			}
		}
		else
		{
			//create command arguments
			String commandArguments=this.createProcessCommandArgumentsForExistingFaxJob(Fax4jExeConstants.RESUME_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE,serverName,faxJobID);
			
			//execute process
			this.executeProcess(FaxActionType.RESUME_FAX_JOB,commandArguments);
		}
	}
	
	/**
	 * This function will cancel an existing fax job.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 */
	private void winCancelFaxJob(String serverName,int faxJobID)
	{
		if(this.useJNI)
		{
			synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
			{
				//pre native call
				this.preNativeCall();
				
				//invoke native
				WindowsFaxClientSpi.cancelFaxJobNative(serverName,faxJobID);
			}
		}
		else
		{
			//create command arguments
			String commandArguments=this.createProcessCommandArgumentsForExistingFaxJob(Fax4jExeConstants.CANCEL_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE,serverName,faxJobID);
			
			//execute process
			this.executeProcess(FaxActionType.CANCEL_FAX_JOB,commandArguments);
		}
	}

	/**
	 * This function returns the fax job status.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 * @return	The fax job status
	 */
	private String winGetFaxJobStatus(String serverName,int faxJobID)
	{
		String status=null;
		if(this.useJNI)
		{
			synchronized(WindowsFaxClientSpi.NATIVE_LOCK)
			{
				//pre native call
				this.preNativeCall();
				
				//invoke native
				status=WindowsFaxClientSpi.getFaxJobStatusNative(serverName,faxJobID);
			}
		}
		else
		{
			//create command arguments
			String commandArguments=this.createProcessCommandArgumentsForExistingFaxJob(Fax4jExeConstants.GET_STATUS_ACTION_FAX4J_EXE_COMMAND_LINE_ARGUMENT_VALUE,serverName,faxJobID);
			
			//execute process
			status=this.executeProcess(FaxActionType.GET_FAX_JOB,commandArguments);
		}
		
		return status;
	}
	
	/**
	 * This function sets the debug mode of the native layer.
	 * 
	 * @param 	debug
	 * 			True to print debug messages, otherwise false
	 */
	private static native void setDebugModeNative(boolean debug);
	
	/**
	 * This function will submit a new fax job.<br>
	 * The fax job ID will be returned by this method.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param	targetAddress
	 * 			The fax job target address
	 * @param	targetName
	 * 			The fax job target name
	 * @param	senderName
	 * 			The fax job sender name
	 * @param	fileName
	 * 			The file to fax
	 * @return	The fax job ID (null in case of an error)
	 */
	private static native int submitFaxJobNative(String serverName,String targetAddress,String targetName,String senderName,String fileName);
	
	/**
	 * This function will suspend an existing fax job.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 */
	private static native void suspendFaxJobNative(String serverName,int faxJobID);
	
	/**
	 * This function will resume an existing fax job.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 */
	private static native void resumeFaxJobNative(String serverName,int faxJobID);
	
	/**
	 * This function will cancel an existing fax job.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 */
	private static native void cancelFaxJobNative(String serverName,int faxJobID);

	/**
	 * This function returns the fax job status.
	 * 
	 * @param 	serverName
	 * 			The fax server name
	 * @param 	faxJobID
	 * 			The fax job ID
	 * @return	The fax job status
	 */
	private static native String getFaxJobStatusNative(String serverName,int faxJobID);
}